<?php

namespace Modules\ResourcesIntegration\Controllers;

require_once dirname(__DIR__) . "/models/Resources.php";
require_once dirname(__DIR__) . "/Parsedown.php";

use App\Controllers\Controller;
use PD\Parsedown;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Modules\ResourcesIntegration\Models\Resource;
use Slim\Exception\{
    HttpNotFoundException,
    HttpForbiddenException
};

class ResourcesController extends Controller
{
    public function resourceraw(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $resourceall =  Resource::where([['id', $args['resource']]])->first();

        if ($this->auth->hasPermission('resources_integration_view_protected') || $this->auth->isOwner()) {
            $resource = $resourceall;
        } else {
            $resource = Resource::where([
                ['id', $args['resource']],
                ['protected', false]
            ])->first();
        }

        if (!$resource) {
            $resourceall =  Resource::where([['resource_pretty_link', $args['resource']]])->first();

            if ($this->auth->hasPermission('resources_integration_view_protected') || $this->auth->isOwner()) {
                $resource = $resourceall;
            } else {
                $resource = Resource::where([
                    ['resource_pretty_link', $args['resource']],
                    ['protected', false]
                ])->first();;
            }
        }

        if (!$resource) {
            if (!$resourceall) {
                throw new HttpNotFoundException($request);
            }
            throw new HttpForbiddenException($request);
        }

        // Ignore the next line of code as the error is not valid
        //
        return $response->withJson($resource)->withHeader('Content-Type', 'application/json');
    }

    public function resource(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $resourceall =  Resource::where([['id', $args['resource']]])->first();

        if ($this->auth->hasPermission('resources_integration_view_protected') || $this->auth->isOwner()) {
            $resource = $resourceall;
        } else {
            $resource = Resource::where([
                ['id', $args['resource']],
                ['protected', false]
            ])->first();
        }

        if (!$resource) {
            $resourceall =  Resource::where([['resource_pretty_link', $args['resource']]])->first();

            if ($this->auth->hasPermission('resources_integration_view_protected') || $this->auth->isOwner()) {
                $resource = $resourceall;
            } else {
                $resource = Resource::where([
                    ['resource_pretty_link', $args['resource']],
                    ['protected', false]
                ])->first();;
            }
        }

        if (!$resource) {
            if (!$resourceall) {
                throw new HttpNotFoundException($request);
            }
            throw new HttpForbiddenException($request);
        }

        $this->view->getEnvironment()->addGlobal('data', [
            'title' => $resource['title'] ?: $resource['navbar_name'],
            'header' => $resource['title'],
            'logo_url' => $resource['logo_url'],
            'rtype' => $resource['rtype'],
            'sourcestyle' => $resource['sourcestyle'],
            'html' => $resource["rtype"] === "Markdown" ? (new Parsedown)->text($resource['text']) : $resource["text"]
        ]);



        return $this->view->render($response, 'markdownpage.twig');
    }
}
