<?php

namespace Modules\ResourcesIntegration\Controllers;

require_once dirname(__DIR__) . "/models/Resources.php";

use App\Controllers\Controller;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Modules\ResourcesIntegration\Models\Resource;

class ResourcesEditorController extends Controller
{
    public function menu(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->view->getEnvironment()->addGlobal('data', [
            'title' => 'Resources'

        ]);

        return $this->view->render($response, 'resourceseditor.twig');
    }

    public function gethandler(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // Check if user has view protected resources permission
        if ($this->auth->hasPermission('resources_integration_view_protected') || $this->auth->isOwner()) {
            $resources = Resource::all();
        } else {
            $resources = Resource::where('protected', false)->get();
        }

        // Make the key of each resource the resource's id
        $resources = $resources->keyBy('id');

        // Return the resources as JSON
        return $response->withJson($resources)->withHeader('Content-Type', 'application/json');
    }

    public function posthandler(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // Get all post data
        $post_data = $request->getParsedBody()['data'];

        // Get only the ids of all resources from the database
        $resource_ids = Resource::all()->pluck('id');

        // Loop through each resource
        foreach ($post_data as $resource) {
            // Get resource
            $resource_model = Resource::find($resource['id']);

            // If resource doesn't exist, create it
            if (!$resource_model) {
                $resource_model = new Resource;
            }

            // Remove the id from $resource_ids
            $resource_ids = $resource_ids->filter(function ($value, $key) use ($resource) {
                return $value != $resource['id'];
            });

            // Update the resource
            $resource_model->title = $resource['title'];
            $resource_model->logo_url = $resource['logo_url'];
            $resource_model->protected = $resource['protected'];
            $resource_model->text = $resource['text'];
            $resource_model->navbar_name = $resource['navbar_name'];
            $resource_model->navbar_icon = $resource['navbar_icon'];
            $resource_model->navbar_category = $resource['navbar_category'];
            $resource_model->resource_pretty_link = $resource['resource_pretty_link'];
            $resource_model->resource_linking = $resource['resource_linking'];
            $resource_model->rtype = $resource['rtype'];
            $resource_model->updated_at = date('Y-m-d H:i:s');
            $resource_model->sourcestyle = $resource['sourcestyle'];
            if (!$resource_model->id) {
                $resource_model->created_at = date('Y-m-d H:i:s');
            }
            $resource_model->id = $resource['id'];

            // Save resource
            $resource_model->save();
        }

        // Delete all resources that are not in the post data
        Resource::whereIn('id', $resource_ids)->delete();

        return $response->withJson(['message' => "Resources updated"]);
    }
}
