<?php

namespace Modules\ResourcesIntegration\Providers;

require_once dirname(__DIR__) . '/models/Resources.php';

use App\Providers\ModuleServiceProvider;
use App\Services\Auth;
use Modules\ResourcesIntegration\Models\Resource;

class ResourcesServiceProvider extends ModuleServiceProvider
{
    public function boot(): void
    {
        $this->loadRoutesFrom($this->module->getPath('routes.php'));
        $this->loadViewsFrom($this->module->getPath('resources/views'));
        $this->loadMigrationsFrom($this->module->getPath('database/migrations'));

        $this->permissions([
            'resources_integration_view_protected' => [
                'title' => 'View Resources',
                'description' => 'Allow the role to view protected resources.',
            ]
        ]);
        $this->navbarLinks([
            [
                'icon' => 'fas fa-boxes-alt',
                'name' => 'Resources',
                'url' => '/admin/resources',
                'admin_dropdown' => true,
            ],
        ]);

        // Create a ConfigRepository, Dispatcher and LoggerInterface instance for Auth
        $auth = app(Auth::class);
        if ($auth->isOwner() || $auth->hasPermission("resources_integration_view_protected")) {
            $searchterms = [['navbar_category', '!=', 'Hidden']];
        } else {
            $searchterms = [['protected', false], ['navbar_category', '!=', 'Hidden']];
        }
        

        try {
            $resources = Resource::where($searchterms)->get();
        } catch (\Exception $e) {
            $resources = [];
        }

        $navbarLinks = [
            'Links' => [
                'icon' => 'fas fa-link',
                'links' => []
            ],
            'Resources' => [
                'icon' => 'fas fa-boxes-alt',
                'links' => []
            ],
            'Rules' => [
                'icon' => 'fas fa-file-alt',
                'links' => []
            ],
            'Changelogs' => [
                'icon' => 'fas fa-history',
                'links' => []
            ],
            'Moderator' => [
                'icon' => 'fas fa-id-badge',
                'links' => []
            ],
        ];

        foreach ($resources as $resource) {
            if ($resource->navbar_category == 'Navbar') {
                if ($resource["resource_pretty_link"]) {
                    $link = '/resources/' . $resource["resource_pretty_link"];
                } else {
                    $link = '/resources/' . $resource["id"];
                }
                $this->navbarLinks([
                    [
                        'icon' => $resource["navbar_icon"],
                        'name' => $resource["navbar_name"],
                        'url' => $link,
                    ],
                ]);
            } else {
                if ($resource["resource_pretty_link"]) {
                    $link = '/resources/' . $resource["resource_pretty_link"];
                } else {
                    $link = '/resources/' . $resource["id"];
                }
                $navbarLinks[$resource->navbar_category]['links'][] = [
                    'icon' => $resource["navbar_icon"],
                    'name' => $resource["navbar_name"],
                    'url' => $link,
                ];
            }
        };

        foreach ($navbarLinks as $category => $joyte) {
            if ($joyte['links']) {
                $this->navbarLinks([
                    [
                        'icon' => $joyte['icon'],
                        'name' => $category,
                        'nested_links' => $joyte['links'],
                    ],
                ]);
            }
        }
    }
}
