function setMaxLength(id, limit) {
    $("#" + id).on('keydown paste', (e) => {
        var allowedKeys = false;

        if (e.type === 'keydown') {
            allowedKeys = (
                e.which === 8 ||  /* BACKSPACE */
                e.which === 35 || /* END */
                e.which === 36 || /* HOME */
                e.which === 37 || /* LEFT */
                e.which === 38 || /* UP */
                e.which === 39 || /* RIGHT*/
                e.which === 40 || /* DOWN */
                e.which === 46 || /* DEL*/
                e.ctrlKey === true && e.which === 65 || /* CTRL + A */
                e.ctrlKey === true && e.which === 88 || /* CTRL + X */
                e.ctrlKey === true && e.which === 67 || /* CTRL + C */
                e.ctrlKey === true && e.which === 86 || /* CTRL + V */
                e.ctrlKey === true && e.which === 90    /* CTRL + Z */
            )
        }
    
        if (e.type === 'paste') {
            setTimeout(function () {
                $(e.target).text($(e.target).text().slice(0, limit));
            });
        }

        if (!allowedKeys && $(e.target).text().length >= limit) {
            e.preventDefault();
        }
    });
}

function createChangelogHTML(group_name) {
    var html = /*html*/`
        <li id="<RANDOMID>" style="list-style: none; margin-top: 1rem;" class="card-body">
            <div style="display: flex; height: 2rem; margin-top: 0.5rem; justify-content: space-between; align-items: center;">
                <div style="display: flex; flex-direction: column;">
                    <h1 contenteditable id="<RANDOMID>_title" style="font-size: 1.5rem; margin: 0;"><GROUPNAME></h1>
                </div>
                <span class="cg-edit-element" style="display: flex; width: 3rem; justify-content: space-between;">
                    <i onclick="removeChangelogGroup('<RANDOMID>')" class="fas fa-xmark cursor-pointer"></i>
                    <i onclick="createChangeLogTopic('<RANDOMID>')" class="fas fa-plus cursor-pointer"></i>
                </span>
            </div>
            <ul id="<RANDOMID>_topiclist" class="topiclist" style="margin-top: 1rem; list-style: none; padding: 0.5rem;">
            </ul>
        </li>
    `;
    id = Math.random().toString(36).substring(2, 9);
    html = html.replace(/<RANDOMID>/g, "changelog_" + id);
    html = html.replace(/<GROUPNAME>/g, group_name ? group_name : "Changelog");
    return [html, id];
}

function addChangelogGroup() {
    document.getElementById("create-changelog-modal").style.display = ""
    document.getElementById("modal_changelog_groupname").focus();
}

function addChangelogGroupConfirm() {
    let htmlid = createChangelogHTML(
        document.getElementById("modal_changelog_groupname").value
    )
    cancelAddChangelogGroup();
    document.getElementById("resource_text").insertAdjacentHTML("afterBegin", htmlid[0]);
    setMaxLength("changelog_" + htmlid[1] + "_title", 20);
    $("#changelog_" + htmlid[1] + "_title").on("keydown", () => {
        updateChangelogs();
    });
    updateChangelogs();
}

function cancelAddChangelogGroup() {
    document.getElementById("create-changelog-modal").style.display = "none"
    document.getElementById("modal_changelog_groupname").value = ""
}

function removeChangelogGroup(id) {
    document.getElementById(id).remove();
    updateChangelogs();
}

function removeChangelogTopic(id) {
    document.getElementById(id).remove();
    updateChangelogs();
}

function createChangeLogTopic(id) {
    let element = document.getElementById(id + "_topiclist");
    let html = /*html*/`
        <li id="<RANDOMID>" style="display: flex; margin-top: 0.5rem; margin-bottom: 0.5rem; padding-left: 1.5rem; align-items: center; justify-content: space-between;">
            <div style="display: flex">
                <select id="<RANDOMID>_select" style="width: 4rem;" class="form-select cg-edit-element fontawesome-font">
                    <option value='&#x2b;'>&#x2b;</option>
                    <option value='&#xf068;'>&#xf068;</option>
                    <option value='&#xf0ad;'>&#xf0ad;</option>
                    <option value='&#xf151;'>&#xf151;</option>
                    <option value='&#xf188;'>&#xf188;</option>
                </select>
                <h3 id="<RANDOMID>_icon" class="fontawesome-font" style="margin: 0; margin-left: 0.6rem; font-size: 1.5rem; display: flex; align-items: center;">&#x2b;</h3>
                <h3 contenteditable id="<RANDOMID>_topicname" style="font-size: 1rem; margin: 0; margin-left: 1rem; display: flex; align-items: center;"><TOPICNAME></h3>
            </div>
            <i onclick="removeChangelogTopic('<RANDOMID>')" class="fas fa-xmark cursor-pointer cg-edit-element" style="margin-right: 0.5rem;"></i>
        </li>
    `;
    id = "topiclist_" + Math.random().toString(36).substring(2, 9);
    html = html.replace(/<RANDOMID>/g, id);
    html = html.replace(/<TOPICNAME>/g, "Topic " + (element.childElementCount + 1));
    element.insertAdjacentHTML("beforeend", html);
    setMaxLength(id + "_topicname", 80);

    window.setTimeout(() => {
        $("#" + id + "_select").on("change", () => {
            document.getElementById(id + "_icon").innerHTML = document.getElementById(id + "_select").value;
            updateChangelogs();
        });
    }, 100);

    updateChangelogs();
}

function updateChangelogs() {
    // Remove all the newlines and `    ` from the changelog
    document.getElementById("resource_text").innerHTML = document.getElementById("resource_text").innerHTML.replace(/\n/g, "").replace(/    /g, "");;
    $(".topiclist").sortable({update: updateChangelogs})
    update();
    window.setTimeout(() => {
        document.querySelectorAll(".topiclist").forEach(function(element) {
            element.querySelectorAll("li").forEach(function(element) {
                // Get the select element
                var select = element.querySelector("select");
                // Get the h3 element with the class fontawesome-font
                var h3 = element.querySelector("h3.fontawesome-font");
                
                $("#" + select.id).on("change", () => {
                    h3.innerHTML = select.value;
                    updateChangelogs();
                    $(".topiclist").each(function(index, element) {
                        $(element).find("li").each(function(index, element) {
                            // Get the icon with the fontawesome-font class
                            var icon = $(element).find("h3").first()[0];
                            var select = $(element).find("select").first()[0];
                            // Set the value of the select option to the innerHTML of the icon
                            select.value = icon.innerHTML;
                        })
                    })
                });
            });
        });
        $(".topiclist").each(function(index, element) {
            $(element).find("li").each(function(index, element) {
                // Get the icon with the fontawesome-font class
                var icon = $(element).find("h3").first()[0];
                var select = $(element).find("select").first()[0];
                // Set the value of the select option to the innerHTML of the icon
                select.value = icon.innerHTML;
            })
        })
    }, 100);
}