document.addEventListener('keydown', e => {
    if (e.ctrlKey && e.key === 's') {
        e.preventDefault();
        saveResources();
    }
});

var resourcesDict = {};
var formDisabled = false;
QUILL = '';

// Randomize an 5 digit number with no leading zeros
function randomNumber() {
    let number = Math.floor(Math.random() * 50000);
    if (number.toString().length == 5 && !(number in resourcesDict)) {
        return number;
    } else {
        return randomNumber();
    }
}

function Resource(id, title, logo_url, protected, text, navbar_name, navbar_icon, navbar_category, resource_pretty_link, resource_linking, rtype, sourcestyle) {
    this.id = id;
    this.title = title;
    this.logo_url = logo_url;
    this.protected = protected;
    this.text = text;
    this.navbar_name = navbar_name;
    this.navbar_icon = navbar_icon;
    this.navbar_category = navbar_category;
    this.resource_pretty_link = resource_pretty_link;
    this.resource_linking = resource_linking;
    this.rtype = rtype;
    this.sourcestyle = sourcestyle;

    this.default = function () {
        this.id = randomNumber();
        this.title = "";
        this.logo_url = "";
        this.protected = false;
        this.text = "";
        this.navbar_name = "";
        this.navbar_icon = "";
        this.navbar_category = "Hidden";
        this.resource_pretty_link = "";
        this.resource_linking = false;
        this.rtype = "";
        this.sourcestyle = "";
    }

    this.view = function () {
        viewResource(this.id);
    }

    this.json = function () {
        return {
            'id': this.id,
            'title': this.title,
            'logo_url': this.logo_url,
            'protected': this.protected,
            'text': this.text,
            'navbar_name': this.navbar_name,
            'navbar_icon': this.navbar_icon,
            'navbar_category': this.navbar_category,
            'resource_pretty_link': this.resource_pretty_link,
            'resource_linking': this.resource_linking,
            'rtype': this.rtype,
            'sourcestyle': this.sourcestyle
        }
    }
}

function exportResources() {
    var resources = {};
    for (var key in resourcesDict) {
        resources[key] = resourcesDict[key].json();
    }
    return resources;
}

function generateResource(name, active, id) {
    var default_active_resource_html = '<span id=<ID> onclick="viewResource(<ID>)" class="badge fs-6 title text-white" style="cursor: pointer; background-color: <COLOR>; margin-top: 0.5rem;"><TEXT>\n</span>';
    if (active) {
        active = 'rgb(143, 161, 175)'
    } else {
        active = 'rgb(58, 65, 71)'
    }
    var resource = default_active_resource_html.replace(/<COLOR>/g, active);
    var resource = resource.replace(/<TEXT>/g, name);
    var resource = resource.replace(/<ID>/g, id);
    return resource;
}

function toggleDisabledForm(disabled) {
    var form = document.getElementsByTagName('form')[1];
    if (form.disabled == disabled) {
        return;
    }
    form.disabled = disabled;

    // Create list of all possible form elements
    var list = ['input', 'textarea', 'button', 'select'];
    // Loop form elements and disable them
    list.forEach(function (element) {
        Array.from(form.getElementsByTagName(element)).forEach(function (input) {
            input.disabled = !input.disabled;
        });
        Array.from(document.getElementById("buttons").getElementsByTagName(element)).forEach(function (input) {
            input.disabled = !input.disabled;
        });
    });
}

function deleteFormData() {
    var form = document.getElementsByTagName('form')[1];
    document.getElementById('resource_link').target = "";
    document.getElementById('resource_link').href = "#";
    form.reset();
}

function updateResources() {
    try {
        var cid = document.getElementById('resource_id').innerHTML.replace(/#/g, '');
    } catch {
        var cid = "";
    }
    var resourcesHTML = '';
    for (let id in resourcesDict) {
        var resource = resourcesDict[id];
        var resource_html = generateResource(resource.title ? resource.title : (resource.navbar_name ? resource.navbar_name : "Untitled"), cid == resource.id, resource.id);
        resourcesHTML += resource_html;
    }
    document.getElementById('resource_list').innerHTML = resourcesHTML;
}

function toggleButtons(show) {
    var buttons = document.getElementById('buttons');
    if (show != undefined) {
        buttons.style.display = show ? "flex" : "none";
    } else if (buttons.style.display == "none") {
        buttons.style.display = "flex";
    } else {
        buttons.style.display = "none";
    }
}

function createNewResource() {
    newresource = new Resource();
    newresource.default();
    resourcesDict[newresource.id] = newresource;
    resourcesDict[newresource.id].title = document.getElementById('modal_createnew_title').value;
    resourcesDict[newresource.id].rtype = document.getElementById('modal_createnew_type').value;
    viewResource(newresource.id);
    toggleCreateModal();
}

function quillImageHandler() {
    var range = this.quill.getSelection();
    var value = prompt('Please enter the image url here.');
    if (value) {
        this.quill.insertEmbed(range.index, 'image', value, Quill.sources.USER);
    }
}

function viewResource(id) {
    var resource = resourcesDict[id];
    var form = document.getElementById("editor");

    toggleButtons(true);
    var url = "/modules/resources-integration?f=/editor/" + resource.rtype + ".html";

    fetch(url).then(function (response) {
        return response.text();
    }).then(function (html) {
        form.innerHTML = html;

        document.getElementById('resource_id').innerHTML = "#" + resource.id;
        document.getElementById('resource_title').value = resource.title;
        document.getElementById('resource_logo_url').value = resource.logo_url;
        if (resource.rtype == "Editor") {
            QUILL = new Quill('#resource_text', {
                theme: 'snow',
                modules: {
                    imageResize: {
                        displaySize: true // default false
                    },
                    toolbar: {
                        container: [
                            [{ 'header': [1, 2, 3, 4, 5, 6, false] }],

                            ['bold', 'italic', 'underline', 'strike'],
                            ['blockquote', 'code-block'],
                            [{ 'list': 'ordered' }, { 'list': 'bullet' }],
                            [{ 'indent': '-1' }, { 'indent': '+1' }],

                            [{ 'color': [] }, { 'background': [] }],
                            ["image", "link"],
                            [{ 'font': [] }],
                            [{ 'align': [] }],

                            ['clean']
                        ],
                        handlers: {
                            image: quillImageHandler
                        }
                    },
                }
            });
            QUILL.root.innerHTML = resource.text;
            QUILL.on("text-change", () => {
                update();
            })
        } else if ($('#resource_text').tagName === "TEXTAREA") {
            document.getElementById("resource_text").value = resource.text;
        } else {
            document.getElementById("resource_text").innerHTML = resource.text;
        }
        document.getElementById('resource_protected').checked = resource.protected;
        document.getElementById("resource_linking").checked = resource.resource_linking;
        document.getElementById('resource_navbar_name').value = resource.navbar_name;
        document.getElementById('resource_navbar_icon').value = resource.navbar_icon;
        document.getElementById('resource_navbar_category').value = resource.navbar_category;
        if (resource.resource_pretty_link) {
            document.getElementById('resource_link').href = '/resources/' + resource.resource_pretty_link;
        } else {
            document.getElementById('resource_link').href = '/resources/' + resource.id;
        }
        document.getElementById('resource_link').target = "_blank";
        document.getElementById("resource_pretty_link").value = resource.resource_pretty_link;
        try {
            document.getElementById('resource_sourcestyle').checked = resource.sourcestyle;
        } catch {
            // Do nothing
        }
        updateResources();
        update();
        window[resource.rtype]();
    }).catch(function (error) {
        alert(error);
    });
}

function updateResource() {
    var id = document.getElementById('resource_id').innerHTML.replace(/#/g, '');
    var resource = resourcesDict[id];
    if (!resource) {
        return;
    }
    resource.id = id;
    resource.title = document.getElementById('resource_title').value;
    resource.logo_url = document.getElementById('resource_logo_url').value;
    if (document.getElementById("resource_text").classList.contains("ql-snow")) {
        resource.text = QUILL.root.innerHTML;
    } else if ($('#resource_text').prop("tagName") === "TEXTAREA") {
        resource.text = document.getElementById('resource_text').value;
    } else {
        resource.text = document.getElementById('resource_text').innerHTML;
    }
    resource.protected = document.getElementById('resource_protected').checked;
    resource.navbar_name = document.getElementById('resource_navbar_name').value;
    resource.navbar_icon = document.getElementById('resource_navbar_icon').value;
    resource.navbar_category = document.getElementById('resource_navbar_category').value;
    resource.resource_pretty_link = document.getElementById('resource_pretty_link').value;
    resource.resource_linking = document.getElementById('resource_linking').checked;
    try {
        resource.sourcestyle = document.getElementById('resource_sourcestyle').checked;
    } catch {
        resource.sourcestyle = false;
    }
    return resource;
}

function update() {
    updateResource();
    updateResources();
}

window.request = function (url, method, data) {
    if (method == "GET") {
        return fetch(url, { method: "GET" })
    } else {
        return fetch(url, { method: method, headers: { 'Content-Type': 'application/json' }, body: JSON.stringify(data) });
    }
}

function deleteResource() {
    document.getElementById("delete-modal").style.display = ""
}

function deleteResourceConfirmed() {
    var cid = document.getElementById('resource_id').innerHTML.replace(/#/g, '');
    var resource = resourcesDict[cid];
    delete resourcesDict[cid];
    updateResources();
    document.getElementById("editor").innerHTML = "";
    toggleButtons(false);
    deleteResourceCancel();
}

function deleteResourceCancel() {
    document.getElementById("delete-modal").style.display = "none";
}

function saveResources() {
    document.getElementById("loading-modal").style.display = "";
    request("/resources", "POST", { "data": exportResources() }).then(function (response) {
        return response.json();
    }).then(function (data) {
        document.getElementById("loading-modal").style.display = "none";
        document.getElementById("save-modal").style.display = "";
        window.setTimeout(() => {
            document.getElementById("save-modal").style.display = "none";
        }, 1500);
    })
}

function toggleCreateModal() {
    var element = document.getElementById('create-modal');

    if (element.style.display == "none") {
        element.style.display = "grid";
        document.body.style.overflow = "hidden";
        document.getElementById("modal_createnew_title").focus();
    } else {
        element.style.display = "none";
        document.body.style.overflow = "auto";
        document.getElementById("modal_createnew_title").value = "";
        document.getElementById("modal_createnew_type").value = "Editor";
    }
}

function getResources() {
    // Get resources from server
    request("/resources", "GET", "").then(function (response) {
        try {
            r = response
            return response.json();
        }
        catch (e) {
            console.log(r.text())
            return { "data": "There was an error getting the resources, please contact the administrator." };
        }
    }).then(function (data) {
        if (!data["error"]) {

            // Turn resource into Resource object
            for (var resource in data) {
                resource = data[resource];
                var resource = new Resource(
                    resource["id"],
                    resource["title"],
                    resource["logo_url"],
                    resource["protected"],
                    resource["text"],
                    resource["navbar_name"],
                    resource["navbar_icon"],
                    resource["navbar_category"],
                    resource["resource_pretty_link"],
                    resource["resource_linking"],
                    resource["rtype"],
                    resource["sourcestyle"]
                );
                if (!resource.rtype) {
                    resource.rtype = "Markdown";
                }
                resourcesDict[resource.id] = resource;
            }

            updateResources();
        } else {
            alert(data["message"]);
            console.log(data["message"])
            console.log(data["error"])
        }
    });

    return resourcesDict;
}

window.addEventListener('load', () => {
    document.getElementById("modal_createnew_title").onkeyup = function (e) {
        if (e.key == "Enter") {
            createNewResource();
        } else if (e.key == "Escape") {
            toggleCreateModal();
        }
    }
    getResources();
});